package ru.myitschool.work.ui.login

import android.os.Bundle
import android.text.Editable
import android.text.InputType
import android.text.TextWatcher
import android.util.Log
import android.view.View
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import dagger.hilt.android.AndroidEntryPoint
import ru.myitschool.work.R
import ru.myitschool.work.databinding.FragmentLoginBinding
import ru.myitschool.work.utils.collectWhenStarted
import ru.myitschool.work.utils.visibleOrGone
import ru.myitschool.work.utils.AuthPreferences
import kotlinx.coroutines.launch
import androidx.lifecycle.lifecycleScope

@AndroidEntryPoint
class LoginFragment : Fragment(R.layout.fragment_login) {
    private var _binding: FragmentLoginBinding? = null
    private val binding get() = _binding!!
    private val viewModel: LoginViewModel by viewModels()
    private lateinit var authPreferences: AuthPreferences

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        authPreferences = AuthPreferences(requireContext())

        // Проверка, авторизован ли пользователь
        if (authPreferences.isLoggedIn()) {
            navigateToMainScreen()
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentLoginBinding.bind(view)

        setupUI()
        subscribe()
        setupLoginTextWatcher()
    }

    private fun setupUI() {
        binding.username.apply {
            inputType = InputType.TYPE_CLASS_TEXT
        }

        binding.login.setOnClickListener {
            val username = binding.username.text.toString()
            val password = binding.password.text.toString() // Получаем пароль
            performLogin(username, password) // Передаем пароль в метод performLogin
        }

        binding.loading.visibleOrGone(false)
        binding.error.visibleOrGone(false)
    }

    private fun setupLoginTextWatcher() {
        binding.username.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                val username = s.toString()
                binding.login.isEnabled = username.isNotEmpty() // Кнопка активна, если поле не пустое
            }

            override fun afterTextChanged(s: Editable?) {}
        })
    }

    private fun performLogin(username: String, password: String) {
        lifecycleScope.launch {
            viewModel.authenticate(username, password) // Передаем пароль в метод authenticate
        }
    }

    private fun subscribe() {
        viewModel.state.collectWhenStarted(this) { state ->
            binding.loading.visibleOrGone(false)

            if (state.error != null) {
                binding.error.text = state.error
                binding.error.visibility = View.VISIBLE
            } else if (state.success) {
                binding.error.visibility = View.GONE
                authPreferences.saveLoginState(true)
                authPreferences.saveLogin(binding.username.text.toString()) // Сохраняем логин
                Toast.makeText(context, "Авторизация прошла успешно", Toast.LENGTH_SHORT).show()
                navigateToMainScreen()
            }
        }
    }

    private fun navigateToMainScreen() {
        try {
            findNavController().apply {
                popBackStack(R.id.mainFragment, false)
                navigate(R.id.mainFragment)
            }
        } catch (e: Exception) {
            Log.e("LF", "Nav_err", e)
            Toast.makeText(context, "Ошибка перехода", Toast.LENGTH_SHORT).show()
        }
    }

    override fun onDestroyView() {
        _binding = null
        super.onDestroyView()
    }
}